/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file SocketExample1.cpp.cpp
 *\date  Created on: 05-11-2009
 *\author  Author: Claudio Salvadori
 *\brief   A simple server in the internet domain using TCP.*/


/*************************************************************************/
/* 	    A simple server in the internet domain using TCP.		 */
/*  		The port number is passed as an argument.		 */
/*************************************************************************/

#include <sys/types.h> 
#include <sys/socket.h>
#include <stdlib.h>											
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <iostream>
#include <string.h>
#include <cstdio>

//Ack value
#define ACK	"1"	
//Nak value
#define NAK	"0"	

//Escape character
#define ESC	0x20

//Start character
#define START	0x02

//End character
#define END	0x03

using namespace std;
int main(int argc, char *argv[])
{
#ifdef DAQSOCKET1
	unsigned int counter = 0;
	int sockfd, newsockfd, portno, clilen;
	char buffer[256];
	struct sockaddr_in serv_addr, cli_addr;
	int n;
	
	//Checking the port number
	if (argc < 2) {
		cout << "Synapsis: ./exe/SocketExample <Port.Number> " <<  endl;
		exit(1);
	}

	//Open a socket
	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if (sockfd < 0) {
		cerr << "ERROR opening socket" << endl;
 		exit(1);
	}
	
	//Fill the server address structure
	bzero((char *) &serv_addr, sizeof(serv_addr));
	portno = atoi(argv[1]);
	serv_addr.sin_family = AF_INET;		//Internet socket
	serv_addr.sin_addr.s_addr = INADDR_ANY;	//Connect to any client
	serv_addr.sin_port = htons(portno);	//Port number

	//Binding operation
	if (bind(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0){
		cerr << "ERROR on binding" << endl;
		exit(1);
	}

	printf("Waiting for connection... \n");

	//Listening operation
	listen(sockfd,5);
	clilen = sizeof(cli_addr); 

	//Accepting a client
	newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr, (socklen_t*)&clilen);
	if (newsockfd < 0){
		cerr << "ERROR on accept" << endl;
		exit(1);
	}

	printf("Connected! \n");

	//Reading the handshaking word
	bzero(buffer,256);
	n = read(newsockfd,buffer,255);
	if (n < 0){
		cerr << "ERROR reading from socket" << endl;
		exit(1);	
	}
	
	printf("Here is the massage: %s\n",buffer);

	//Checking if the handshaking packet is correct and answer
	if(!strcmp (buffer, "1")){
		n = write(newsockfd, ACK, 1);
		if (n < 0){
			cerr << "ERROR writing to socket" << endl;
			exit(1);
		}
	}else{
		n = write(newsockfd, NAK, 1);
		if (n < 0){
			cerr << "ERROR writing to socket" << endl;
			exit(1);
		}
	}
	
	
	//Infinite cycle for sending packets
	while(1){
		counter++;
		#ifdef DEBUG
		cerr << "Actual counter value: " << counter << endl;
		#endif
		unsigned int length_p = 0;
		unsigned char packet[50];
			
		packet[length_p++] = START;
		
		packet[length_p++] = (unsigned char)'D';
		
		packet[length_p++] = (unsigned char)'A';
		
		packet[length_p++] = (unsigned char)'Q';
		
		packet[length_p++] = (unsigned char)'*';
		
		if(((unsigned char)((counter >> 24) & 0xFF)) == START){
			packet[length_p++] = ESC;
			packet[length_p++] = START ^ ESC;
		}else if(((unsigned char)((counter >> 24) & 0xFF)) == END){
			packet[length_p++] = ESC;
			packet[length_p++] = packet[length_p] ^ ESC;
		}else if(((unsigned char)((counter >> 24) & 0xFF)) == ESC){
			packet[length_p++] = ESC;
			packet[length_p++] = ESC ^ ESC;
		}else{
			packet[length_p++] = (unsigned char)((counter >> 24) & 0xFF);
		}
			
		if(((unsigned char)((counter >> 16) & 0xFF)) == START){
			packet[length_p++] = ESC;
			packet[length_p++] = START ^ ESC;
		}else if(((unsigned char)((counter >> 16) & 0xFF)) == END){
			packet[length_p++] = ESC;
			packet[length_p++] = END ^ ESC;
		}else if(((unsigned char)((counter >> 16) & 0xFF)) == ESC){
			packet[length_p++] = ESC;
			packet[length_p++] = ESC ^ ESC;
		}else{
			packet[length_p++] = (unsigned char)((counter >> 16) & 0xFF);
		}
		

		if(((unsigned char)((counter >> 8) & 0xFF)) == START){
			packet[length_p++] = ESC;
			packet[length_p++] = START ^ ESC;
		}else if(((unsigned char)((counter >> 8) & 0xFF)) == END){
			packet[length_p++] = ESC;
			packet[length_p++] = END ^ ESC;
		}else if(((unsigned char)((counter >> 8) & 0xFF)) == ESC){
			packet[length_p++] = ESC;
			packet[length_p++] = ESC ^ ESC;
		}else{
			packet[length_p++] = (unsigned char)((counter >> 8) & 0xFF);
		}
		
		if(((unsigned char)((counter) & 0xFF)) == START){
			packet[length_p++] = ESC;
			packet[length_p++] = START ^ ESC;
		}else if(((unsigned char)((counter) & 0xFF)) == END){
			packet[length_p++] = ESC;
			packet[length_p++] = END ^ ESC;
		}else if(((unsigned char)((counter) & 0xFF)) == ESC){
			packet[length_p++] = ESC;
			packet[length_p++] = ESC ^ ESC;
		}else{
			packet[length_p++] = (unsigned char)((counter) & 0xFF);
		}
		
		packet[length_p++] = END;
		
		n = write(newsockfd, packet, length_p);
		if (n < 0){
			cerr << "ERROR writing to socket" << endl;
			exit(1);
		}else{
			cerr << "Packet size: " << dec << length_p << endl;
			cerr << "Packet sent: ";
			for(unsigned int k = 0; k < length_p; k++){
				cerr << hex << (int)packet[k] << ", ";
			}
			cerr << endl;
		}
	} 


	return 0; 
#else
	cout << "Error: this executable has not been compiled with the DAQSOCKET1 symbol." <<  endl;
	cout << "Recopile with: make socket1" <<  endl;	
	exit(1);
#endif  
}
