/*
Copyright (C) 2009 -  Claudio Salvadori
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file CFormatter.cpp
 *\brief  Formatting functions development
 */
#include <iostream>
#include "CFormatter.h"

#define U_SIZE 	8 // size of a memory unit in number of bit (8bit = 1byte)

//constructor
CFormatter::CFormatter(std::vector<SData>* data): f_Data(data){}

//destructor
CFormatter::~CFormatter(){
	f_Metadata = NULL;
	f_Data = NULL;
}

//init method
f_ret CFormatter::init(std::vector<struct SMData>* metadata){

	f_Metadata = metadata;

	return createDataV();
}

//create data vector
f_ret CFormatter::createDataV(){
	struct SData elem;

	if(f_Metadata == NULL)
		return F_META_ERR;

	for(unsigned int i = 0; i < f_Metadata->size(); i++){

		//fix the type
		elem.d_Name = (*f_Metadata)[i].md_Name;
		elem.d_Type = (*f_Metadata)[i].md_Type;
		switch (elem.d_Type){
			case T_INT8:
			elem.d_Size = (*f_Metadata)[i].md_Size / INT8_S;
			elem.d_Data = new char[elem.d_Size];
			break;

			case T_UINT8:
			elem.d_Size = (*f_Metadata)[i].md_Size / INT8_S;
			elem.d_Data = new unsigned char[elem.d_Size];
			break;
			
			case T_INT16:
			elem.d_Size = (*f_Metadata)[i].md_Size / INT16_S;
			elem.d_Data = new short[elem.d_Size];
			break;
			
			case T_UINT16:
			elem.d_Size = (*f_Metadata)[i].md_Size / INT16_S;
			elem.d_Data = new unsigned short[elem.d_Size];
			break;

			case T_INT32:
			elem.d_Size = (*f_Metadata)[i].md_Size / INT32_S;
			elem.d_Data = new int[elem.d_Size];
			break;

			case T_UINT32:
			elem.d_Size = (*f_Metadata)[i].md_Size / INT32_S;
			elem.d_Data = new unsigned int[elem.d_Size];
			break;

			case T_INT64:
			elem.d_Size = (*f_Metadata)[i].md_Size / INT64_S;
			elem.d_Data = new long[elem.d_Size];
			break;
	
			case T_UINT64:
			elem.d_Size = (*f_Metadata)[i].md_Size / INT64_S;
			elem.d_Data = new unsigned long[elem.d_Size];
			break;
			
			case T_FLOAT32:
			elem.d_Size = (*f_Metadata)[i].md_Size / FLOAT32_S;
			elem.d_Data = new float[elem.d_Size];
			break;
			
			case T_FLOAT64:
			elem.d_Size = (*f_Metadata)[i].md_Size / FLOAT64_S;
			elem.d_Data = new double[elem.d_Size];
			break;

			case T_NOTYPE:
			return F_META_ERR;
			break;
		}
		
		//fix the endianess
		elem.d_End = (*f_Metadata)[i].md_End;
				
		f_Data->push_back(elem);
	}

	return F_RET_OK;

}

//format data and return formatted data vector
f_ret CFormatter::getData(unsigned char* pkt){

	if(f_Data == NULL)
		return F_D_NULL_ERR;

	if(f_Data->size() == 0)
		return F_D_EMPTY_ERR;
	
	//k is a counter to read metadata and data vector
	for(unsigned int k = 0; k < f_Metadata->size(); k++)
	{
		//create the vector of the subset of the stream vector to pass to the formatter function		
		unsigned char * subset = NULL;
		subset = new unsigned char[(*f_Metadata)[k].md_Size];
		//Copying the correct subvector from stream vector
		memcpy(subset, (unsigned char*) (pkt + (*f_Metadata)[k].md_Pos), (*f_Metadata)[k].md_Size);
			
		switch((*f_Data)[k].d_Type){
			case T_INT8:
			(*f_Data)[k].d_Data = getInt8((unsigned char*)subset, (*f_Data)[k].d_Size, (*f_Data)[k].d_End);
			break;

			case T_UINT8:
			(*f_Data)[k].d_Data = getUInt8((unsigned char*)subset, (*f_Data)[k].d_Size, (*f_Data)[k].d_End);
			break;
			
			case T_INT16:
			(*f_Data)[k].d_Data = getInt16((unsigned char*)subset, (*f_Data)[k].d_Size, (*f_Data)[k].d_End);
			break;

			case T_UINT16:
			(*f_Data)[k].d_Data = getUInt16((unsigned char*)subset, (*f_Data)[k].d_Size, (*f_Data)[k].d_End);
			break;

			case T_INT32:
			(*f_Data)[k].d_Data = getInt32((unsigned char*)subset, (*f_Data)[k].d_Size, (*f_Data)[k].d_End);
			break;

			case T_UINT32:
			(*f_Data)[k].d_Data = getUInt32((unsigned char*)subset, (*f_Data)[k].d_Size, (*f_Data)[k].d_End);
			break;

			case T_INT64:
			(*f_Data)[k].d_Data = getInt64((unsigned char*)subset, (*f_Data)[k].d_Size, (*f_Data)[k].d_End);
			break;
	
			case T_UINT64:
			(*f_Data)[k].d_Data = getUInt64((unsigned char*)subset, (*f_Data)[k].d_Size, (*f_Data)[k].d_End);
			break;
			
			case T_FLOAT32:
			(*f_Data)[k].d_Data = getFloat32((unsigned char*)subset, (*f_Data)[k].d_Size, (*f_Data)[k].d_End);
			break;
			
			case T_FLOAT64:
			(*f_Data)[k].d_Data = getFloat64((unsigned char*)subset, (*f_Data)[k].d_Size, (*f_Data)[k].d_End);
			break;
			
			case T_NOTYPE:
			return F_META_ERR;
			break;
		
		}
	}
	return F_RET_OK;
}

//integer formatter
int* CFormatter::getInt32(unsigned char *v, unsigned long _size, endianess_type _endianess){
	int* help = NULL;
	help = new int[_size];
	
	switch(_endianess){
		case E_BIG:
		for(unsigned long i = 0; i < _size; i++){
			help[i] = 0;
			for(int n = 0; n < INT32_S; n++)
				help[i] += (int)(*(v + i * INT32_S + n)  << ((INT32_S - n - 1) * U_SIZE));
		}
		break;
		
		case E_LITTLE:
		for(unsigned long i = 0; i < _size; i++){
			help[i] = 0;
			for(int n = 0; n < INT32_S; n++)
				help[i] += (int)(*(v + i * INT32_S + n)  << (n * U_SIZE));
		}
		break;
	}
	return help;
}

//unsigned integer formatter
unsigned int* CFormatter::getUInt32(unsigned char *v, unsigned long _size, endianess_type _endianess){
	unsigned int* help = NULL;
	help = new unsigned int[_size];
		
	switch(_endianess){
		case E_BIG:
		for(unsigned long i = 0; i < _size; i++){
			help[i] = 0;
			for(int n = 0; n < INT32_S; n++)
				help[i] += (unsigned int)(*(v + i * INT32_S + n)  << ((INT32_S - n - 1) * U_SIZE));
		}
		break;
		
		case E_LITTLE:
		for(unsigned long i = 0; i < _size; i++){
			help[i] = 0;
			for(int n = 0; n < INT32_S; n++)
				help[i] += (unsigned int)(*(v + i * INT32_S + n)  << (n * U_SIZE));	
		}
		break;
	}
	return help;
}

//char formatter
char* CFormatter::getInt8(unsigned char *v, unsigned long _size, endianess_type _endianess){
	char* help = NULL;
	help = new char[_size];
	
	switch(_endianess){
		case E_LITTLE:
		case E_BIG:
		for(unsigned long i = 0; i < _size; i++){
			help[i] = (char)(*(v + i * INT8_S));
		}
		break;
	}
	return help;
}

//unsigned char formatter
unsigned char* CFormatter::getUInt8(unsigned char *v, unsigned long _size, endianess_type _endianess){
	unsigned char* help = NULL;
	help = new unsigned char[_size];
	
	switch(_endianess){
		case E_BIG:
		case E_LITTLE:
		for(unsigned long i = 0; i < _size; i++){
			help[i] = (unsigned char)(*(v + i * INT8_S));	
		}
		break;
	}
	return help;
}

//short formatter
short* CFormatter::getInt16(unsigned char *v, unsigned long _size, endianess_type _endianess){
	short* help = NULL;
	help = new short[_size];
	
	switch(_endianess){
		case E_BIG:
		for(unsigned long i = 0; i < _size; i++){
			help[i] = 0;
			for(int n = 0; n < INT16_S; n++)
				help[i] += (short)(*(v + i * INT16_S + n)  << ((INT16_S - n - 1) * U_SIZE));
		}
		break;
		
		case E_LITTLE:
		for(unsigned long i = 0; i < _size; i++){
			help[i] = 0;
			for(int n = 0; n < INT16_S; n++)
				help[i] += (short)(*(v + i * INT16_S + n)  << (n * U_SIZE));	
		}
		break;
	}
	return help;
}


//unsigned short formatter
unsigned short* CFormatter::getUInt16(unsigned char *v, unsigned long _size, endianess_type _endianess){
	unsigned short* help = NULL;
	help = new unsigned short[_size];
	
	switch(_endianess){
		case E_BIG:
		for(unsigned long i = 0; i < _size; i++){
			help[i] = 0;
			for(int n = 0; n < INT16_S; n++)
				help[i] += (unsigned short)(*(v + i * INT16_S + n)  << ((INT16_S - n - 1) * U_SIZE));
		}
		break;
		
		case E_LITTLE:
		for(unsigned long i = 0; i < _size; i++){
			help[i] = 0;
			for(int n = 0; n < INT16_S; n++)
				help[i] += (unsigned short)(*(v + i * INT16_S + n)  << (n * U_SIZE));
		}
		break;
	}
	return help;
}

//long formatter
long* CFormatter::getInt64(unsigned char *v, unsigned long _size, endianess_type _endianess){
	long* help = NULL;
	help = new long[_size];
	
	switch(_endianess){
		case E_BIG:
		for(unsigned long i = 0; i < _size; i++){
			help[i] = 0;
			for(int n = 0; n < INT64_S; n++)
				help[i] += (long)(*(v + i * INT64_S + n)  << ((INT64_S - n - 1) * U_SIZE));
		}
		break;
		
		case E_LITTLE:
		for(unsigned long i = 0; i < _size; i++){
			help[i] = 0;
			for(int n = 0; n < INT64_S; n++)
				help[i] += (long)(*(v + i * INT64_S + n)  << (n * U_SIZE));
		}
		break;
	}
	return help;
}

//unsigned long formatter
unsigned long* CFormatter::getUInt64(unsigned char *v, unsigned long _size, endianess_type _endianess){
	unsigned long* help = NULL;
	help = new unsigned long[_size];
	
	switch(_endianess){
		case E_BIG:
		for(unsigned long i = 0; i < _size; i++){
			help[i] = 0;
			for(int n = 0; n < INT64_S; n++)
				help[i] += (unsigned long)(*(v + i * INT64_S + n)  << ((INT64_S - n - 1) * U_SIZE));	
		}
		break;
		
		case E_LITTLE:
		for(unsigned long i = 0; i < _size; i++){
			help[i] = 0;
			for(int n = 0; n < INT64_S; n++)
				help[i] += (unsigned long)(*(v + i * INT64_S + n)  << (n * U_SIZE));	
		}
		break;
	}
	return help;
}

//float formatter
float* CFormatter::getFloat32(unsigned char *v, unsigned long _size, endianess_type _endianess){
	float* help = NULL;
	help = new float[_size];

	switch(_endianess){
		case E_BIG:
		for(unsigned long i = 0; i < _size; i++){
			unsigned int tmp = 0;
			for(int n = 0; n < FLOAT32_S; n++)
				tmp += (unsigned int) (*(v + i * FLOAT32_S + n)  << ((FLOAT32_S - n - 1) * U_SIZE));
			help[i] = *((float*) &tmp);	
		}
		break;
		
		case E_LITTLE:
		for(unsigned long i = 0; i < _size; i++){
			unsigned int tmp = 0;
			for(int n = 0; n < FLOAT32_S; n++)
				tmp += (unsigned int) (*(v + i * FLOAT32_S + n)  << (n * U_SIZE));
			help[i] = *((float*) &tmp);	
		}
		break;
	}
	return help;
}

//double formatter
double* CFormatter::getFloat64(unsigned char *v, unsigned long _size, endianess_type _endianess){
	double* help = NULL;
	help = new double[_size];
	
	switch(_endianess){
		case E_BIG:
		for(unsigned long i = 0; i < _size; i++){
			unsigned long tmp = 0;
			help[i] = 0;
			for(int n = 0; n < FLOAT64_S; n++)
				tmp += (unsigned long)(*(v + i * FLOAT64_S + n)  << ((FLOAT64_S - n - 1) * U_SIZE));
			help[i] += *((double*) &tmp);
		}
		break;
		
		case E_LITTLE:
		for(unsigned long i = 0; i < _size; i++){
			unsigned long tmp = 0;
			help[i] = 0;
			for(int n = 0; n < FLOAT64_S; n++)
				tmp += (unsigned long)(*(v + i * FLOAT64_S + n)  << (n * U_SIZE));
			help[i] += *((double*) &tmp);
		}
		break;
	}
	return help;
}



